Package: BayesianMCPMod
Title: Simulate, Evaluate, and Analyze Dose Finding Trials with
        Bayesian MCPMod
Version: 1.3.2
Authors@R: c(
    person("Boehringer Ingelheim Pharma GmbH & Co. KG", role = c("cph", "fnd")),
    person("Stephan", "Wojciekowski", , "stephan.wojciekowski@boehringer-ingelheim.com", role = c("aut", "cre")),
    person("Lars", "Andersen", , "lars.andersen@boehringer-ingelheim.com", role = "aut"),
    person("Jonas", "Schick", , "jonas.schick@boehringer-ingelheim.com", role = "ctb"),
    person("Sebastian", "Bossert", , "sebastian.bossert@boehringer-ingelheim.com", role = "aut")
  )
Description: Bayesian MCPMod (Fleischer et al. (2022)
    <doi:10.1002/pst.2193>) is an innovative method that improves the
    traditional MCPMod by systematically incorporating historical data,
    such as previous placebo group data. This package offers functions
    for simulating, analyzing, and evaluating Bayesian MCPMod trials with
    normally and binary distributed endpoints. It enables the assessment of trial
    designs incorporating historical data across various true
    dose-response relationships and sample sizes. Robust mixture prior
    distributions, such as those derived with the Meta-Analytic-Predictive
    approach (Schmidli et al. (2014) <doi:10.1111/biom.12242>), can be
    specified for each dose group.  Resulting mixture posterior
    distributions are used in the Bayesian Multiple Comparison Procedure
    and modeling steps. The modeling step also includes a weighted model
    averaging approach (Pinheiro et al. (2014) <doi:10.1002/sim.6052>).
    Estimated dose-response relationships can be bootstrapped and
    visualized.
License: Apache License (>= 2)
URL: https://boehringer-ingelheim.github.io/BayesianMCPMod/,
        https://github.com/Boehringer-Ingelheim/BayesianMCPMod
BugReports: https://github.com/Boehringer-Ingelheim/BayesianMCPMod/issues
Depends: R (>= 4.2)
Imports: checkmate, DoseFinding (>= 1.1-1), dplyr, ggplot2, logistf,
        methods, nloptr, RBesT, stats, tidyr
Suggests: clinDR, doFuture, future.apply, kableExtra, knitr,
        MCPModPack, reactable, rmarkdown, spelling, testthat (>=
        3.0.0), tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-14 05:46:06 UTC; wojcie
Author: Boehringer Ingelheim Pharma GmbH & Co. KG [cph, fnd],
  Stephan Wojciekowski [aut, cre],
  Lars Andersen [aut],
  Jonas Schick [ctb],
  Sebastian Bossert [aut]
Maintainer: Stephan Wojciekowski <stephan.wojciekowski@boehringer-ingelheim.com>
Repository: CRAN
Date/Publication: 2026-05-14 14:00:19 UTC
Built: R 4.4.3; ; 2026-05-14 14:31:59 UTC; unix
