# Generated by roxygen2: do not edit by hand

S3method(plot,modelFits)
S3method(predict,modelFits)
S3method(print,BayesianMCP)
S3method(print,BayesianMCPMod)
S3method(print,modelFits)
S3method(print,postList)
S3method(summary,postList)
export(assessDesign)
export(getBootstrapQuantiles)
export(getBootstrapSamples)
export(getContr)
export(getCritProb)
export(getESS)
export(getMED)
export(getModelFits)
export(getPosterior)
export(performBayesianMCP)
export(performBayesianMCPMod)
export(simulateData)
