<!-- badges: start -->
[![Linux](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/linux.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/linux.yml)
[![Windows](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/windows.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/windows.yml)
[![Macos](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/macos.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/macos.yml)
[![Codecov test coverage](https://codecov.io/gh/Boehringer-Ingelheim/BayesianMCPMod/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Boehringer-Ingelheim/BayesianMCPMod?branch=main)
<!-- badges: end -->

# The `BayesianMCPmod` package

Simulate, analyze, and evaluate Bayesian MCPMod trials with normally and binary distributed endpoints.

## Installation

Install the package from CRAN using

```{r}
install.packages("BayesianMCPMod")
```

The development version can be installed from this repository.

```{r}
# install.packages("remotes")
remotes::install_github("https://github.com/Boehringer-Ingelheim/BayesianMCPmod")
```


## Documentation

The package documentation is hosted [here](https://boehringer-ingelheim.github.io/BayesianMCPMod/).

